#!/usr/bin/env python2
# -*- coding: utf-8 -*-
# coding = utf-8
'''
Тест для проверки комплекта драйверов
'''

import ctypes as C
import mmap
import os
import random
import sys
import time


class spc_caps(C.Structure):
    _fields_ = [("name", C.c_char * 16),
            ("core_nr", C.c_uint),
            ("flags", C.c_uint),
            ("shmem_offs", C.c_ulong),
            ("shmem_size", C.c_ulong),
            ("pmr_nr", C.c_uint),
            ("pmr_offs", C.c_ulong * 8),
            ("pmr_sizes", C.c_ulong * 8),
            ]

_IO_WRITE = 1
_IO_READ = 2

_IOC_NRSHIFT = 0
_IOC_TYPESHIFT = 8
_IOC_SIZESHIFT = 16
_IOC_DIRSHIFT = 30

_IOC = lambda io_dir, io_type, io_nr, io_size:\
                (io_dir << _IOC_DIRSHIFT) | (io_type << _IOC_TYPESHIFT) |\
                (io_nr << _IOC_NRSHIFT) | (io_size << _IOC_SIZESHIFT)

_IOR = lambda io_type, io_nr, io_size: _IOC(_IO_READ, io_type, io_nr, io_size)
_IOW = lambda io_type, io_nr, io_size: _IOC(_IO_WRITE, io_type, io_nr, io_size)
_IOWR = lambda io_type, io_nr, io_size: _IOC(_IO_READ | _IO_WRITE, io_type, io_nr, io_size)

SPC_IOCTL_MAGIC = 0x9d

SPC_IOCTL_GET_CAPS = _IOR(SPC_IOCTL_MAGIC, 0, C.sizeof(spc_caps))

libc = C.CDLL('libc.so.6')
libc.mmap.argtypes = [C.c_void_p, C.c_size_t, C.c_int, C.c_int, C.c_int, C.c_long]
libc.mmap.restype = C.c_long
libc.munmap.argtypes = [C.c_long, C.c_long]
libc.ioctl.argtypes = [C.c_int, C.c_int, C.c_void_p]

class spc_device:
    @staticmethod
    def enumerate(name):
        return len([f for f in os.listdir('/dev') if name in f])

    def __init__(self, name, nr):
        self.nr = nr
        self.f = open('/dev/%s-%u' % (name, nr), 'rw')
        self.caps = spc_caps()
        libc.ioctl(self.f.fileno(), SPC_IOCTL_GET_CAPS, C.addressof(self.caps))
        self.mmapres = libc.mmap(0, self.caps.pmr_sizes[0],
                            mmap.PROT_READ | mmap.PROT_WRITE,
                            mmap.MAP_PRIVATE,
                            self.f.fileno(), self.caps.pmr_offs[0])
        self.io = (C.c_int*self.caps.pmr_sizes[0]).from_address(self.mmapres)
        return

    def __del__(self):
        self.f.close()

    def read(self, offset):
        return C.c_uint.from_buffer(self.io, offset).value    

    def write(self, value, offset):
        C.c_uint.from_buffer(self.io, offset).value = value
        return


if __name__ == '__main__':
    errcount = 0

    for j in ("ums8", "ums7"):
        count = spc_device.enumerate(j)
        print 'found', j, 'entries:', count
        for i in xrange(0, count):
            print 'trying device %s-%u' % (j, i)
            s = spc_device(j, i)
            ver = s.read(0x0)
            addr = 0x2000600  # Temp. sensor on bridge FPGA
            temp = (((float(int(s.read(addr)) >> 4)) * 503.975) / 4096.0 - 273.5)
            print 'Temper:', int(temp)
            print 'FW ver:', ver

            testval1 = random.randint(0, 2**31)
            s.write(testval1, 0x4)
            testval2 = s.read(0x4)
            if testval1 == testval2:
                print 'Scratch check OK'
            else:
                print 'Scratch check failed:', testval1, testval2
                errcount += 1
            time.sleep(1)

    sys.exit(errcount)
